'''
Module for PageWriter, which used for creating pages in Wiki
'''
from typing import Dict, Tuple
from .person import Person
from jinja2 import Environment, FileSystemLoader
from gql import Client, gql
from gql.transport.requests import RequestsHTTPTransport
from configparser import SectionProxy


def _normalise_str(s: str) -> (str):
    '''
    Method for normalise string to send it in GraphQl
    '''
    return s.replace('\\', '\\\\').replace('\n', '\\n').replace('\r', '')


class PageWriter():
    '''
    Class for connecting to wiki and creating new pages
    '''

    def __init__(
        self, wiki_config: SectionProxy,
        person_template: str = 'person_page.tmpl',
        list_template: str = 'list_page.tmpl',
        query_template: str = 'query.tmpl'
    ) -> (None):
        '''
        Constructor for PageWrite class
        '''
        env = Environment(loader=FileSystemLoader('./templates/'))
        self._user_template = env.get_template(person_template)
        self._list_template = env.get_template(list_template)
        self._query_template = env.get_template(query_template)
        self._path = wiki_config['path']
        sample_transport = RequestsHTTPTransport(
            url=wiki_config['url'],
            use_json=True,
            headers={
                'Authorization': 'Bearer ' + wiki_config['token'],
                'Content-type': 'application/json'
            }
        )
        self._client = Client(
            transport=sample_transport,
            fetch_schema_from_transport=True,
        )

    def write_person_page(self, person: Person) -> (Tuple[bool, str]):
        '''
        Method to write person's page
        '''
        content = self._user_template.render(person=person)
        query_text = self._query_template.render(
            content=_normalise_str(content),
            path=self._path + person.username,
            title=person.full_name
        )
        resp = self._client.execute(
            gql(query_text)
        )['pages']['create']['responseResult']

        link = ''
        if resp['succeeded']:
            link = './' + person.username
        person.set_link(link)
        return resp['succeeded'], resp['message']

    def write_list_page(
        self,
        persons_by_role: Dict,
        title: str = 'Список участников'
    ) -> (Tuple[bool, str]):
        '''
        Method to write list of members page
        '''
        content = self._list_template.render(persons_by_role=persons_by_role)
        query_text = self._query_template.render(
            content=_normalise_str(content),
            path=self._path + 'list',
            title=title
        )
        resp = self._client.execute(
            gql(query_text)
        )['pages']['create']['responseResult']
        return resp['succeeded'], resp['message']
